# Copyright 1999-2025. WebPros International GmbH. All rights reserved.

# Unfortunately, mailman3 postorius does not offer a method to send confirmation
# emails from the virtual domains using request data.
# Therefore, we need to create a subclass of DefaultAccountAdapter from django allauth
# with overridden get_from_email() method, as explained in the allauth source code,
# to ensure that confirmation emails are sent from the correct domain.
# Further information can be found here: https://github.com/pennersr/django-allauth/blob/main/allauth/account/adapter.py#L43-L47

from django.conf import settings
from django.contrib.sites.models import Site
from allauth.account.adapter import DefaultAccountAdapter


class PleskAccountAdapter(DefaultAccountAdapter):
    def get_from_email(self):
        """
        This is a hook to set the 'from' email address for sending emails
        """
        domain = self.request.get_host()
        return f"postorius@{domain}"


def create_site_if_not_exists(domain):
    if Site.objects.filter(domain=domain).exists():
        return

    free_id = 1
    if Site.objects.all():
        free_id = Site.objects.all().order_by("-id").first().id + 1

    Site.objects.create(id=free_id, domain=domain, name=domain)


class PleskMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        # We need to manually search for the site by its domain in the request
        # because django does not automatically handle this.
        # Once we find the site, we must associate it with the request object
        # and also update the local instance of the settings.SITE_ID variable
        target_domain = request.META.get("HTTP_HOST")
        if not Site.objects.filter(domain=target_domain).exists():
            # We do not add missing sites to the database during a Plesk update.
            # As a result, some older sites might be missing, and we must create them on the go.
            create_site_if_not_exists(target_domain)

        request.site = Site.objects.get(domain=target_domain)
        request.site_id = request.site.id
        settings.SITE_ID = request.site.id

        return self.get_response(request)
